////////////////////////////////////////////////////////////////////////////////
// Filename: labelled_transition.cc                                           //
// Purpose : Implementation of class Unlabelled_Transition                    //
//           A class that provides labelled transitions                       //
////////////////////////////////////////////////////////////////////////////////
// Author: Gerald Anleitner                                                   //
// Theme.: 2                                                                  //
// Group.: 5 (Gerald Anleitner, Markus Keidl, Stefan Seltzsam)                //
//                                                                            //
// Date of creation: 14.05.1996        Date of last change: 04.07.1996        //
////////////////////////////////////////////////////////////////////////////////

#ifndef _LABELLED_TRANSITION_CC_
#define _LABELLED_TRANSITION_CC_

#include <labelled_transition.hh>

Labelled_Transition::Labelled_Transition()
{
  // default constructor
#ifdef DEBUG
  cout << "Labelled_Transition::Labelled_Transition()" << endl;
#endif
}

Labelled_Transition::Labelled_Transition(const Labelled_Transition& given_labelled_transition):
  label(given_labelled_transition.label)
{
  // copy-constructor
#ifdef DEBUG  
  cout << "Labelled_Transition::Labelled_Transition(const Labelled_Transition& given_labelled_transition)" << endl;
#endif
  targetstate=given_labelled_transition.targetstate;
  type = given_labelled_transition.type;
}

Labelled_Transition::Labelled_Transition(State* given_state, Input_Atom given_input_atom):
  label(given_input_atom)
{
  // initialize with label and targetstate
#ifdef DEBUG
  cout << "Labelled_Transition::Labelled_Transition(State* given_state, Input_Atom given_input_atom)" << endl;
#endif
  targetstate=given_state;
  type = DEFAULT;
}

Labelled_Transition::Labelled_Transition(Input_Atom given_input_atom):
  label(given_input_atom)
{
  // initialize only with label
#ifdef DEBUG
  cout << "Labelled_Transition::Labelled_Transition(Input_Atom given_input_atom)" << endl;
#endif
  type = DEFAULT;
}

Labelled_Transition::Labelled_Transition(State* given_state, Input_Atom given_input_atom, int trans_type):
  label(given_input_atom)
{
  // initialize with label, targetstate and type of transition
#ifdef DEBUG
  cout << "Labelled_Transition::Labelled_Transition(State* given_state, Input_Atom given_input_atom)" << endl;
#endif
  targetstate=given_state;
  type = trans_type;
}

Labelled_Transition::Labelled_Transition(Input_Atom given_input_atom, int trans_type):
  label(given_input_atom)
{
  // initilaize only with label and type of transition
#ifdef DEBUG
  cout << "Labelled_Transition::Labelled_Transition(Input_Atom given_input_atom)" << endl;
#endif
  type = trans_type;
}

Labelled_Transition::~Labelled_Transition()
{
  // comitting suicide ...
#ifdef DEBUG
  cout << "Labelled_Transition::Labelled_Transition::~Labelled_Transition()" << endl;
#endif
}

void Labelled_Transition::set_label(const Input_Atom& given_input_atom)
{
  // set the label of the transition
#ifdef DEBUG
  cout << "virtual void Labelled_Transition::set_label(const Input_Atom& given_input_atom)" << endl;
#endif
  label = given_input_atom;
}

Input_Atom Labelled_Transition::get_label() const
{
  // get the label of the transition
#ifdef DEBUG
  cout << "virtual Labelled_Transition::Input_Atom get_label()" << endl;
#endif
  return label;
}

int Labelled_Transition::get_type()
{
  // get the type of the transition
  return type;
}

void Labelled_Transition::set_type(int new_type)
{
  // set the type of the transition
  type = new_type;
}

inline void Labelled_Transition::send(Protocol& protocol_list)
{
#ifdef DEBUG
  cout << "virtual void Labelled_Transition::send(Protocollist&)" << endl;  
#endif

  Input_Stream& i_stream = protocol_list.get_Input_Stream();
#ifdef DEBUG
  cout << "Input Atom: " << i_stream().get_Input_Atom() << endl;
#endif

  // do we have a epsilon transition?
  if (label.get_Input_Atom() == "" && type != POINT)
    // yes, we have a epsilon transition!
    targetstate->send(protocol_list);
  else
    // no, we have not a epsilon transition!
    switch (type)
      {
      case DEFAULT:
	// does the input_atom fit the label?
	// Is the end of the input_stream reached?
	if ( !i_stream && i_stream() == label )
	  {
	    // yes!
	    // so proceed with sending...
	    Pix i = i_stream.get_itr();
	    i_stream.next();
	    targetstate->send(protocol_list);
	    i_stream.set_itr(i);
	  }
	break;
      case POINT:
	// Is the end of the input_stream reached?
	if ( !i_stream )
	  {
	    Pix i = i_stream.get_itr();
	    i_stream.next();
	    targetstate->send(protocol_list);
	    i_stream.set_itr(i);
	  }
	break;
      case OR:
	// Is the input_stream reached?
	// Is the actual input_atom of the input_stream in the set of the
	// transitions input_atoms?
	if ( !i_stream && label.get_Input_Atom().contains(i_stream().get_Input_Atom()))
	  {
	    Pix i = i_stream.get_itr();
	    i_stream.next();
	    targetstate->send(protocol_list);
	    i_stream.set_itr(i);
	  }
	break;
      }

}

#endif //_LABELLED_TRANSITION_CC_

// 19.06.1996 Edited line 96: added '!i_stream &&' to prevent errors   




