////////////////////////////////////////////////////////////////////////////////
// Filename: scanner_main.cc                                                  //
// Purpose : controlls the command line parameters and the text scanning      //
// builds up a NFA and and scanns the specified text                          //
////////////////////////////////////////////////////////////////////////////////
// Author: Markus Keidl                                                       //
// Theme.: 2                                                                  //
// Group.: 5 (Gerald Anleitner, Stefan Seltzsam, Markus Keidl)                //
//                                                                            //
// Date of creation: 29.05.1996        Date of last change: 15.07.1996        //
////////////////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <fstream.h>
#include <String.h>
#include <bool.h>
#include "Automaton.hh"

// later: name of language file and input file as command line parameters
int main(int argc, char* argv[])
{
  bool param_error = false;
  bool save_nfa = false;
  bool in_file = true;
  bool out_file = false;
  int offset = 1;
  
  // interpret command line parameters
  if (argc == 1)			    // too less parameters
    param_error = true;
  else
    if (argv[1][0] == '-')		    // save nfa in Sgraph-format
      if (argv[1][1] == 's')
	{
	  save_nfa = true;
	  offset = 3;
	  switch (argc)
	    {
	    case 4:				    // no input file, no output file
	      in_file = false;
	      break;
	    case 5:				    // no output file
	      break;
	    case 6:				    // maximum number of files
	      out_file = true;
	      break;
	    default:			    // too less or too much parameters
	      param_error = true;
	    }
	} 
      else
	{
	  cerr << "wrong option following -" << endl;
	  cerr << "usage:     scanner <grammerdef_file> <sourcecode_file> [<token_file>]" << endl;
	  cerr << "usage: or  scanner -s <Sgraph_file> <grammerdef_file> [<sourcecode_file>] [<token_file>]" << endl;
	  return false;	
	}
    else
      {
	switch (argc)
	  {
	  case 3:				    // no output file
	    break;
	  case 4:				    // maximum number of files
	    out_file = true;
	    break;
	  default:			    // too less or too much parameters
	    param_error = true;
	  }
      }
  if (param_error == true)                 // less than two parameters
    {
      cerr << "wrong number of command line parameters" << endl;
      cerr << "usage:     scanner <grammerdef_file> <sourcecode_file> [<token_file>]" << endl;
      cerr << "usage: or  scanner -s <Sgraph_file> <grammerdef_file> [<sourcecode_file>] [<token_file>]" << endl;
      return false;
    }

  ofstream write_to;
  if (out_file == true)
    {
      write_to.open(argv[offset+2]);
      if (!write_to)
	{
	  cerr << argv[offset+2] << ": unable to create file" << endl;
	  return false;
	}
    }

  // create automaton and read Input_Stream
  Automaton nfa;
  Input_Stream text;
  if (!nfa.create_from_regexp(argv[offset]))
    {
      cerr << "### error occured, try again ;)" << endl;
      return false;
    }
  if (save_nfa == true)
    {
      if (!nfa.sgraph(argv[2]))
	{
	  cerr << argv[2] << ": unable to create file" << endl;
	  return false;
	}
    }

  if (in_file == true)
    {
      nfa.optimize_automaton();
      if (!text.read_from_file(argv[offset+1]))
	{
	  cerr << "### error occured, try again ;)" << endl;
	  return false;
	}

      Protocol hit(text);
      while (text.is_empty() != true)
	{
	  // send rest stream to start state of NFA
	  hit.clear();
	  nfa.match(text,hit);
	  // print the scanned token
	  if (hit.get_length() > 0)
	    {
	      if (hit.get_token() != "WHITESPACE")
		if (out_file == true)
		  write_to << hit << flush;
		else
		  cout << hit << flush;
	      text.cut_off(hit.get_length()); // Reststring ermitteln
	    }
	  else  // no token defined for this input_atom or epsilon scanned
	    if (hit.get_token() != "")
	      {
		cerr << endl << argv[0] << ": program terminated because of epsilon loop" << endl;
		return false;
	      }
	    else
	      {
		cerr << endl << argv[0] << ": character with no token in " << argv[2] << endl;
		return false;
	      }
	}
    }
  if (out_file == true)
    write_to << endl;
  else
    cout << endl;
  return true;
}

