////////////////////////////////////////////////////////////////////////////////
// Filename: state.hh                                                         //
// Purpose : defines the class 'State'                                        //
// Baseclass for several types of states needed for building up the AUTOMATON //
////////////////////////////////////////////////////////////////////////////////
// Author: Stefan Seltzsam                                                    //
// Theme.: 2                                                                  //
// Group.: 5 (Gerald Anleitner, Stefan Seltzsam, Markus Keidl)                //
//                                                                            //
// Date of creation: 22.05.1996        Date of last change: 22.05.1996        //
////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------

#ifndef __STATE_HH__
#define __STATE_HH__

#include <DLList.h>
#include <Protocol.hh>
#include <transition.hh>

class Transition;

//------------------------------------------------------------------------------
// class State - abstract class needed for Unlabelled_State
//------------------------------------------------------------------------------

class State
{

public:

  // add a transition to transition_list
  // you must create a transition and take care for his life. Don't delete the object by yourself,
  // if you add it to this list. I will destroy it, if it is no longer used.
  virtual void add_transition(Transition* trans) = 0;

  // remove a Transition from the DLLlist. Fisrt the Transition is destroyed.
  virtual void remove_transition(Transition* trans) = 0;
  
  // set internal iterator to first transition of transition_list
  virtual void first() = 0;

  // set internal iterator to next transition of transition_list  
  virtual void next() = 0;

  // clear transition_list and delete all transitions
  virtual void clear() = 0;

  // get the actual transition, which the internal iterator is pointing at - NULL = End of List
  virtual Transition* operator()() const = 0;

  // forward the Protocol_List to all transitions stored in transition_list
  virtual void send(Protocol& hit) = 0;

  // get token; if state is not a end state, returned token is ""
  virtual String get_token() { return ""; };
protected:
  
  // stores all transitions leaving this state
  DLList<Transition*> transition_list;

  // internal iterator storing the actual poition in the transition_list
  Pix itr;

};

#endif
