////////////////////////////////////////////////////////////////////////////////
// Filename: unlabelled_end_state.cc                                          //
// Purpose : implements the class 'Unlabelled_End_State'                      //
// class for unlabelled end states needed for building up the AUTOMATON       //
////////////////////////////////////////////////////////////////////////////////
// Author: Stefan Seltzsam                                                    //
// Theme.: 2                                                                  //
// Group.: 5 (Gerald Anleitner, Stefan Seltzsam, Markus Keidl)                //
//                                                                            //
// Date of creation: 22.05.1996        Date of last change: 22.05.1996        //
////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------

#include <unlabelled_end_state.hh>

//------------------------------------------------------------------------------
// class Unlabelled_End_State
//------------------------------------------------------------------------------

// contructors & destructor
Unlabelled_End_State::Unlabelled_End_State() 
{
#ifdef DEBUG
  cout << "Unlabelled_End_State::Unlabelled_End_State()" << endl;
#endif
  epsilon_loop_detection = this;
};

Unlabelled_End_State::~Unlabelled_End_State() 
{
#ifdef DEBUG
  cout << "Unlabelled_End_State::~Unlabelled_End_State()" << endl;
#endif
  for (Pix i = transition_list.first(); i != 0; transition_list.next(i))
    delete transition_list(i);
};

Unlabelled_End_State::Unlabelled_End_State(const Unlabelled_End_State& rhs)
{
#ifdef DEBUG
  cout << "Unlabelled_End_State::Unlabelled_End_State(const Unlabelled_End_State& rhs)" << endl;
#endif
  epsilon_loop_detection = rhs.epsilon_loop_detection;
  transition_list = rhs.transition_list;
  itr = rhs.itr;
  token = rhs.token;
}

Unlabelled_End_State::Unlabelled_End_State(const String& init_token) : token(init_token)
{
#ifdef DEBUG
  cout << "Unlabelled_End_State::Unlabelled_End_State(const String& init_token)" << endl;
#endif
  epsilon_loop_detection = this;
}

// new implemented send method
void Unlabelled_End_State::send(Protocol& hit) 
{
#ifdef DEBUG
  cout << "void Unlabelled_End_State::send(Protocol& hit)" << endl;
#endif

  // Am I in a epsilon Cycle ?
  if (epsilon_loop_detection == hit.get_Input_Stream().get_itr())
    return;
  
  // No, so store actual itr of Input_Stream
  epsilon_loop_detection = hit.get_Input_Stream().get_itr();
  
  // There is something in the scanned input_stream, which was recognized, so add it to hit
  hit.add_hit(token);
  
  // Now, forward hit to all transitions, if there is still a Input_Atom in hit.text, 
  // which was not recognized
  Input_Stream& i_stream = hit.get_Input_Stream();
  if (!i_stream)
    {
      for (Pix i = transition_list.first(); i != 0; transition_list.next(i))
	transition_list(i)->send(hit);
    }
};

void Unlabelled_End_State::set_token(const String& tok)
{
#ifdef DEBUG
  cout << "void Unlabelled_End_State::set_token(const String& tok)" << endl;
#endif
  token = tok;
}

String Unlabelled_End_State::get_token()
{
#ifdef DEBUG
  cout << "String Unlabelled_End_State::get_token()" << endl;
#endif
  return token;
}
